module Creation exposing (..)

import Stats exposing (AttributeCategory)

type alias CreationPoints =
    { freebie : Int
    , spheres : Int
    , attributes : List Priority
    }

type alias Priority =
    { value : Int
    , category : AttributeCategory
    }

new : CreationPoints 
new =
    { freebie = 15
    , spheres = 6
    , attributes =
        [ { value = 7, category = Stats.Physical }
        , { value = 5, category = Stats.Social }
        , { value = 3, category = Stats.Mental }
        ]
    }

cost :
    { sphere : Int
    }
cost = 
    { sphere = 7
    }

buySphere : CreationPoints -> Int -> CreationPoints
buySphere cp value =
    { cp | spheres = cp.spheres - value }

attributeCategoryPriority : List Priority -> AttributeCategory -> Int
attributeCategoryPriority priorities category =
    priorities
    |> List.indexedMap Tuple.pair
    |> List.filter ( \x -> ( Tuple.second x ).category == category )
    |> List.head
    |> Maybe.withDefault ( -1, { value = 0, category = Stats.Physical } )
    |> Tuple.first

priorityAtPosition : List Priority -> Int -> Priority
priorityAtPosition priorities value =
    priorities
    |> List.indexedMap Tuple.pair
    |> List.filter ( \x -> Tuple.first x == value )
    |> List.head
    |> Maybe.withDefault ( -1, { value = 0, category = Stats.Physical } )
    |> Tuple.second

changePriority : CreationPoints -> AttributeCategory -> Int -> CreationPoints
changePriority cp category value =
    let
        priorities : List Priority
        priorities = cp.attributes

        currentPrio : Int
        currentPrio = attributeCategoryPriority priorities category

        unchangedPrio : Int
        unchangedPrio = 3 - ( value + currentPrio )

        newPriorities : List Priority
        newPriorities = 
            if currentPrio == value
            then priorities
            else
                [ ( unchangedPrio, priorityAtPosition priorities unchangedPrio )
                , ( currentPrio, priorityAtPosition priorities value )
                , ( value, priorityAtPosition priorities currentPrio )
                ]
                |> List.sortBy Tuple.first
                |> List.map Tuple.second
    in 
        { cp | attributes = newPriorities }